/* 
 * File:   main.cpp
 * Author: Knoll, Pernkopf, Kulmer
 *
 * Created on Nov 24, 2017
 * Updated Okt 03, 2018
 */
#include <math.h>
#include <iostream>
#include "blockprocessing.h"
#include "blockstructure.h"
#include "FIRfilter.h"
#include "taps.cpp"
#include <portaudio.h>

using namespace std;

#define DECFACT 2
#define INTFACT 2

// collect objects for computation in this struct
struct UserDataExperiment1{
    int checkCallback;
    float sigbufdec[FRAMELENGTH/DECFACT];

    blockstructure block;
};

// the callback function, evaluated at each interrupt
static int paCallback(const void *input, void *output,
    unsigned long framelength,
    const PaStreamCallbackTimeInfo* timeInfo,
    PaStreamCallbackFlags statusFlags,
    void *userdata)
{
    float *in = (float*)input;  
    float *out = (float*)output;
    

    UserDataExperiment1 *ud = (UserDataExperiment1*)userdata;

    int OFFSET=0;
    ud->block.decimate(in, ud->sigbufdec, DECFACT, OFFSET, framelength);
    ud->block.interpolate(ud->sigbufdec, out, INTFACT, OFFSET, framelength/DECFACT);

    ud->checkCallback ++; // report that callback function is working
    return 0;
}

int main(int argc, char *argv[]) {   
    PaStream *stream;
    PaError err;
    int checkCallback_local = 0; // check whether callback function is still active
    int iTimecounter = 0;
    int Ncoeff, Ncoeff1;
    
    
    // --> Experiment one
    UserDataExperiment1 ud; // create objects

    ud.checkCallback = 0; // a watchdog if callback is still working (necessary for RPi) 
    // <-- Experiment one
    
    // the following loop stops and restarts the portaudio stream if it is broken
    do{
        err = Pa_Initialize();
        std::cout << "Initialize portaudio: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_OpenDefaultStream(&stream, 
            NINPUTCHANNELS, 
            NOUTPUTCHANNELS, 
            paFloat32, 
            FS, 
            FRAMELENGTH, 
            paCallback, 
            &ud);
        std::cout << "Open audio stream: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_StartStream(stream);
        std::cout << "Start audio stream: " << Pa_GetErrorText( err ) << std::endl;

        Pa_Sleep(TIMETOSLEEP); // wait until Stream is open
        
        // wait in this loop until DURATION is reached or callback function needs to be restarted
        while(iTimecounter < DURATION-1 && ud.checkCallback != checkCallback_local){ // there was a callback within last second
            checkCallback_local = ud.checkCallback;
            Pa_Sleep(TIMETOSLEEP); // wait a second
            iTimecounter++;
        }
        
        if(iTimecounter != DURATION-1 && ud.checkCallback != checkCallback_local){
            std::cout << "There is a problem with portaudio -> restart" << std::endl;
        }

        err = Pa_StopStream(stream);
        std::cout << "Stop audio stream: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_CloseStream(stream);
        std::cout << "Close audio stream: " << Pa_GetErrorText( err ) << std::endl;
    }while(iTimecounter < DURATION-1);

    Pa_Terminate();
    return 0;    
}
